#include "stdafx.h"
#include "ClippingFilter.h"
#include "ClippingFilterDlg.h"

using namespace _DSHOWLIB_NAMESPACE;


CClippingFilter::CClippingFilter()
{
	m_bEnabled = true;
	m_Threshold = 200;
	m_FillMode = 0;
	m_ClipAboveThreshold = true;
	m_FillColor = RGB(255,0,0);

	// The parameter input and output methods must be members of the class
	addLongParam( L"Threshold", &CClippingFilter::getThreshold, &CClippingFilter::setThreshold );
	addBoolParam( L"Enable",  &CClippingFilter::getEnabled,  &CClippingFilter::setEnabled);
	addBoolParam( L"ClipAboveThreshold",  &CClippingFilter::getClipAboveThreshold,  &CClippingFilter::setClipAboveThreshold);
	addLongParam( L"FillMode",  &CClippingFilter::getFillMode,  &CClippingFilter::setFillMode );
	addLongParam( L"FillColor",  &CClippingFilter::getFillColor,  &CClippingFilter::setFillColor );
}

//////////////////////////////////////////////////////////////////////////

_DSHOWLIB_NAMESPACE::FilterInfo CClippingFilter::getStaticFilterInfo()
{
	static _DSHOWLIB_NAMESPACE::FilterInfo f = { L"Clipping", L"", eFC_GENERIC };
	return f;
}

//////////////////////////////////////////////////////////////////////////
// Implementation of the parameter input and output methods.
/////////////////////////////////////////////////////////////////////////

tFilterError CClippingFilter::setEnabled( bool bVal )
{
	m_bEnabled = bVal;
	return eNO_ERROR;
}
tFilterError CClippingFilter::getEnabled( bool &bVal )
{
	bVal = m_bEnabled;
	return eNO_ERROR;
}

tFilterError CClippingFilter::setThreshold( long lVal )
{
	m_Threshold = lVal;
	return eNO_ERROR;
}

tFilterError CClippingFilter::getThreshold( long &lVal )
{
	lVal = m_Threshold;
	return eNO_ERROR;
}

tFilterError CClippingFilter::setFillMode( long val )
{
	if( val == 0 )
		m_FillMode = 0;
	else
		m_FillMode = 1;

	return eNO_ERROR;
}

tFilterError CClippingFilter::getFillMode( long& val )
{
	val = m_FillMode;

	return eNO_ERROR;
}

tFilterError CClippingFilter::setClipAboveThreshold( bool val )
{
	m_ClipAboveThreshold = val;
	return eNO_ERROR;
}

tFilterError CClippingFilter::getClipAboveThreshold( bool& val )
{
	val = m_ClipAboveThreshold;

	return eNO_ERROR;
}

tFilterError CClippingFilter::setFillColor( long val )
{
	m_FillColor = (COLORREF)val;
	return eNO_ERROR;
}

tFilterError CClippingFilter::getFillColor( long& val )
{
	val = (long)m_FillColor;
	return eNO_ERROR;
}

//////////////////////////////////////////////////////////////////////////
// This method is used by the framework to check whether this filter has a
// property dialog.
//////////////////////////////////////////////////////////////////////////

bool CClippingFilter::hasDialog() const
{
	return true;	// This class has its own property dialog.
}
//////////////////////////////////////////////////////////////////////////
// Show the dialog. This method is called by the framework.
//////////////////////////////////////////////////////////////////////////

bool CClippingFilter::callDialog( HWND hParent )
{
	CClippingFilterDlg dlg( this );
	dlg.show( hParent );

	return true;
}


//////////////////////////////////////////////////////////////////////////
// The array "arr" is filled with the color formats that are supported by
// this filter. If the filter only handels monochrome images,
// then only "eY800" should be added to the array.
//////////////////////////////////////////////////////////////////////////

void CClippingFilter::getSupportedInputTypes( FrameTypeInfoArray& arr ) const 
{
	arr.addFrameType( eY800 );
	arr.addFrameType( eRGB32 );
}
//////////////////////////////////////////////////////////////////////////

bool CClippingFilter::getTransformOutputTypes( const FrameTypeInfo& in_type, FrameTypeInfoArray& out_types ) const 
{
	out_types.addFrameType( MEDIASUBTYPE_RGB32, in_type.dim.cx, in_type.dim.cy );
	
	return true;
};

int calcBrightness( const RGBQUAD& pixel )
{
	return int( 0.30 * pixel.rgbRed + 0.59 * pixel.rgbGreen + 0.11 * pixel.rgbBlue );
}

void generatePattern( RGBQUAD& pixel, int x, int y )
{
	if( ((x / 4) % 2 + (y / 4) % 2) % 2 )
	{
		pixel.rgbBlue = 0;
		pixel.rgbGreen = 0;
		pixel.rgbRed = 0;
		pixel.rgbReserved = 0;
	}	
	else
	{
		pixel.rgbBlue = 255;
		pixel.rgbGreen = 255;
		pixel.rgbRed = 255;
		pixel.rgbReserved = 0;
	}
}

//////////////////////////////////////////////////////////////////////////
// This is the transformation method. If m_bEnabled is true, every pixel of
// the source frame is stored inverted in the destination frame.
// If m_bEnabled is false, the source frame is copied to the
// destination frame without any changes.
//////////////////////////////////////////////////////////////////////////

bool CClippingFilter::transform( const IFrame& src, IFrame& dest )
{
	// If the filter is used in a FrameHandlerSink, and no MemBufferCollection
	// is specified or all MemBuffers are locked, the destination frame pointer is 0.
	if( dest.getPtr() == 0 ) return false;

	int width = src.getFrameType().dim.cx;
	int height = src.getFrameType().dim.cy;

	// Check the input type: If the input type is Y800, the image data has to
	// be converted to RGB32.
	if( src.getFrameType().subtype == MEDIASUBTYPE_Y800 )
	{
		BYTE* pSrc = src.getPtr();
		RGBQUAD* pDest = (RGBQUAD*)dest.getPtr();

		for( int y = 0; y < height; ++y )
		{
			for( int x = 0; x < width; ++x )
			{
				BYTE src = pSrc[x + (height-y-1) * width];
				RGBQUAD& dest = pDest[x + y * width];

				if( !m_bEnabled )
				{
					dest.rgbBlue = dest.rgbGreen = dest.rgbRed = src;
					dest.rgbReserved = 0;
				}
				else
				{
					bool bClip;

					if( m_ClipAboveThreshold )
					{
						bClip = (src > m_Threshold);
					}
					else
					{
						bClip = (src < m_Threshold);
					}

					if( bClip )
					{
						if( m_FillMode == 0 )
						{
							generatePattern( dest, x, y );
						}
						else
						{
							dest.rgbRed = (BYTE)m_FillColor & 0xff;
							dest.rgbGreen = (BYTE)((m_FillColor & 0xff00) >> 8);
							dest.rgbBlue = (BYTE)((m_FillColor & 0xff0000) >> 16);
							dest.rgbReserved = 0;
						}
					}
					else
					{
						dest.rgbBlue = dest.rgbGreen = dest.rgbRed = src;
						dest.rgbReserved = 0;
					}
				}
			}
		}
	}
	else
	{
		RGBQUAD* pSrc = (RGBQUAD*)src.getPtr();
		RGBQUAD* pDest = (RGBQUAD*)dest.getPtr();

		for( int y = 0; y < height; ++y )
		{
			for( int x = 0; x < width; ++x )
			{
				const RGBQUAD& src = pSrc[x + y * width];
				RGBQUAD& dest = pDest[x + y * width];

				if( !m_bEnabled )
				{
					dest = src;
				}
				else
				{
					bool bClip;

					if( m_ClipAboveThreshold )
					{
						bClip = (calcBrightness( src ) > m_Threshold);
					}
					else
					{
						bClip = (calcBrightness( src ) < m_Threshold);
					}

					if( bClip )
					{
						if( m_FillMode == 0 )
						{
							generatePattern( dest, x, y );
						}
						else
						{
							dest.rgbRed = (BYTE)m_FillColor & 0xff;
							dest.rgbGreen = (BYTE)((m_FillColor & 0xff00) >> 8);
							dest.rgbBlue = (BYTE)((m_FillColor & 0xff0000) >> 16);
							dest.rgbReserved = 0;
						}
					}
					else
					{
						dest = src;
					}
				}
			}
		}		
	}


	return true;
}
